/*==============================================================================


       dlg_file - File listing dialog for regular and mmc system

               Copyright  2001 Pankhurst Algorithmics - All Rights Reserved
               visit www.ebmDevMag.com for permissions/restrictions on use

==============================================================================*/
#include <stdlib.h>
#include <string.h>

#include "gui.h"
extern "C"
{
#include "lcd.h"
}
#include "dlg_file.h"
/*==============================================================================

       CFilePickList class

==============================================================================*/
class CFilePickList : public CList
{

  private:

    ebo_enumerator_t **m_entry; // pointer to file list array

    U16 GetRowHeight(S32 row)
    {  return GUI_FontHeight(GetWindow()->GetFont());                     }
    void DrawRow(RECT *rect, S32 row)
    {  GetWindow()->DrawText(m_entry[row]->name.name, rect->x, rect->y);  }

  public:

    CFilePickList(int id,int dx,int dy,int numRows,ebo_enumerator_t **entryList)
     : CList(id,dx,dy,numRows,0),  m_entry(entryList)
    {   }

};
/*==============================================================================

      CPickWindow class

==============================================================================*/
class CPickWindow : public CWindow
{

  private:

    enum  {  ID_PICKWINDOW=100,  ID_LIST,  c_maxEntry=200,  };

    ebo_enumerator_t *m_rec, *m_entry[c_maxEntry];
    int  m_totalEntry;

    S32  MsgHandler(MSG_TYPE type, CViewable *object, S32 data);

    static int SortFunc(const void *a, const void *b);

  public:

    CPickWindow(ebo_enumerator_t *info,const char *ext);
    ~CPickWindow(void);

    static bool Valid;

};
//------------------------------------------------------------------------------
/*static*/ bool CPickWindow::Valid;
//------------------------------------------------------------------------------
CPickWindow::CPickWindow(ebo_enumerator_t *info,const char *ext)
 : CWindow(ID_PICKWINDOW,5,5,LCD_QueryWidth()-10,LCD_QueryHeight()-10,"Open File",FALSE,FALSE,TRUE),
   m_rec(info), // pointer to destination
   m_totalEntry(0)
{
  // set flag so we know when close pressed (nothing returned)
  Valid=false;
  // get list of valid files into m_entry list
  ebo_enumerator_t e;
  e.index=-1; // init search
	while ( ebo_next_object(&e)>=0  && m_totalEntry<c_maxEntry-1 )
  {
    if (0==stricmp(e.name.extension,ext)) // one of ours?
    {
      // add to list
      m_entry[m_totalEntry]=new ebo_enumerator_t;
      memcpy(m_entry[m_totalEntry],&e,sizeof(ebo_enumerator_t));
      ++m_totalEntry;
    }
  }
  // now read from MMC
  e.index=-1; // init search
	while ( ebo_next_xobject(&e)>=0  && m_totalEntry<c_maxEntry-1 )
  {
    if (0==stricmp(e.name.extension,ext)) // one of ours?
    {
      // add to list
      m_entry[m_totalEntry]=new ebo_enumerator_t;
      memcpy(m_entry[m_totalEntry],&e,sizeof(ebo_enumerator_t));
      ++m_totalEntry;
    }
  }
  // check if anything
  if (0==m_totalEntry)
  {
    GUI_Alert(ALERT_WARNING,"No files to load!");
    Close();
    return;
  }
  // sort what we have found
  qsort(m_entry,m_totalEntry,sizeof(m_entry[0]),SortFunc);
  // and now add picklist to window
  RECT rect;
  GetUsableRect(&rect);
  CFilePickList *PickList = new CFilePickList(ID_LIST,rect.width-10,rect.height-10,m_totalEntry,m_entry);
  AddChild(PickList, 5, 5);
  SetFocus(PickList);
}
//------------------------------------------------------------------------------
CPickWindow::~CPickWindow(void)
{
  for (int i=0;i<m_totalEntry;i++)
    delete m_entry[i];
}
//------------------------------------------------------------------------------
/*static*/int CPickWindow::SortFunc( const void *a, const void *b)
{
  // used by qsort
  ebo_enumerator_t *ptrA=*((ebo_enumerator_t**)a);
  ebo_enumerator_t *ptrB=*((ebo_enumerator_t**)b);
  return stricmp( ptrA->name.name, ptrB->name.name );
}
//------------------------------------------------------------------------------
S32 CPickWindow::MsgHandler(MSG_TYPE type, CViewable *object, S32 data)
{
  if ( MSG_ROW_SELECT==type ) // was a row picked?
  {
    memcpy(m_rec,m_entry[data],sizeof(ebo_enumerator_t)); // copy to output
    Valid=true;
    Close(); // one selection immediately exits
  }
	return 1;
}
/*==============================================================================

  OpenDialog - displays files matching ext, and allows picking
               returns entry and 'ok' (true) or nothing and cancel ('false')

==============================================================================*/
bool OpenDialog(ebo_enumerator_t *info,const char *ext)
{
  GUI_EventLoop( new CPickWindow(info,ext) );
  return (CPickWindow::Valid); // set to null if not valid entry
}
//------------------------------------------------------------------------------

